require 'net/http'
#require 'logger'
require 'json'

##
# This class allows to access forum.guns.ru via JSON API
#
# every instance method returns pair (HTTPResponse.code, Hash)
#
class GunsruAPI
    ##
    # Creates new JSON API accessor
    #
    def initialize(base_uri = "forum.guns.ru", api_uri = "/api/1.0/")
        @base_uri = base_uri
        @api_uri = api_uri
        #@logger = Logger.new(STDOUT)
        #@logger.formatter = proc do |sev, date, progname, msg|
            #"#{msg}\n"
        #end
        @etags = {}
    end

    private

    def getResponse(*args)
        callerName = caller[0][/`.*'/][1..-2]
        params = Hash[args.each_slice(2).to_a]
        params[:format] = "json"
        api_call = "#{@api_uri}#{callerName}?#{URI.encode_www_form(params)}"
        http_req = Net::HTTP::Get.new(api_call)
        http_req["If-None-Match"] = @etags[callerName]

        #@logger.debug("sending HTTP::Get to #{api_call}, if-none-macth=#{@etags[callerName]}")
        http = Net::HTTP.new(@base_uri)
        http_resp = http.request(http_req)
        #@logger.debug("got HTTPResponse, code = #{http_resp.code}, etag=#{http_resp["etag"]}")
        @etags[callerName] = http_resp["etag"]
        code = http_resp.code.to_i
        # sometimes gunsru sends bad UTF-8, and it breaks JSON parser
        # i tried Iconv and String.encode, but was not able to solve this problem
        result = JSON.parse(http_resp.body.force_encoding('UTF-8').gsub(/[^[:print:]]/u, ''))["result"] if code == 200
        return code, result
    end

    public

    ##
    # Arguments: none
    # Cacheable: yes
    # Returns: hash
    #   {
    #     timestamp: <unixtime>,
    #     categories: [ {
    #       type: "category",
    #       title: <string>,
    #       id: <integer>,
    #       sections: [ {
    #         type: <string>,
    #         title: <string>,
    #         id: <integer>,
    #         modificationTime: <unixtime>,
    #         categoryId: <integer>
    #   }]}]}
    #
    def getAllCategories()
        getResponse()
    end

    ##
    # Arguments: id, from, to
    # Cacheable: yes
    # Returns: hash
    #   {
    #     timestamp: <unixtime>,
    #     section_id: <unixtime>,
    #     topics: [ {
    #       type: "topic",
    #       topic_type: <string>,
    #       title: <string>,
    #       id: <integer>,
    #       element_number: <integer>,
    #       modificationTime: <unixtime>,
    #       postCount: <integer>,
    #       author: <string>,
    #       lastPost: <string> (author)
    #   }]}
    #
    def getTopicsBySectionId(id, from=0, to=20)
        getResponse('id', id, 'from', from, 'to', to)
    end

    ##
    # Arguments: from, to
    # Cacheable: yes
    # Returns: hash
    #   {
    #     timestamp: <unixtime>,
    #     topics: [ {
    #       type: "topic",
    #       title: <string>,
    #       id: <integer>,
    #       modificationTime: <unixtime>,
    #       postCount: <integer>,
    #       author: <string>,
    #       lastPost: <string> (author)
    #   }]}
    #
    def getActiveTopics(from=0, to=20)
        getResponse('from', from, 'to', to)
    end

    ##
    # Arguments: id, direction, count. Optional: number, start_msg_id
    # Cacheable: yes
    # Returns: hash
    #   {
    #     timestamp: <unixtime>,
    #     topic_id: <integer>,
    #     messages: [ {
    #       type: "post",
    #       body: <string>,
    #       id: <integer>,
    #       element_number: <integer>,
    #       createTime: <unixtime>,
    #       author: { [
    #         nickname: <string>,
    #         id: <integer>,
    #         userpic: [ { ... }],
    #       ]}
    #   }]}
    #
    def getPostsByTopicId(id, direction=1, count=1024)
        getResponse('id', id, 'direction', direction, 'count', count)
    end

    def getLinkedSectionsById(*args)
        return -1, {}
        getResponse
    end
    def addTopicToFavourites()
        return -1, {}
        getResponse
    end
    def getLinkedTopicsById()
        return -1, {}
        getResponse
    end
    def addMessageToTopic()
        return -1, {}
        getResponse
    end
    def getSectionById()
        return -1, {}
        getResponse
    end
    def getTopicById()
        return -1, {}
        getResponse
    end
    def getPostById()
        return -1, {}
        getResponse
    end


    # news & last topics
    def getLastTopics()
        return -1, {}
        getResponse
    end
    def getNews()
        return -1, {}
        getResponse
    end

    # personal messages
    def getUserProfileById()
        return -1, {}
        getResponse
    end
    def getConversations()
        return -1, {}
        getResponse
    end
    def getConversationById()
        return -1, {}
        getResponse
    end
    def getFullMessageById()
        return -1, {}
        getResponse
    end
    def getNewMessageCount()
        return -1, {}
        getResponse
    end

    # favorite sections manipulation
    def getFavoritesSections()
        return -1, {}
        getResponse
    end
    def getFavoriteTopics()
        return -1, {}
        getResponse
    end
    def removeTopicFromFavourites()
        return -1, {}
        getResponse
    end
    # push notifications
    def registerIosDevice()
        return -1, {}
        getResponse
    end
    def unRegisterIosDevice()
        return -1, {}
        getResponse
    end
    def logout()
        return -1, {}
        getResponse
    end

end


